@@ -242,13 +242,27 @@ bool UCS4String::UCS4StringValue::set (const iterator & ci, UCS4Char ucs4)
       throw std::out_of_range("UCS4String::iterator - iterator not for this string!");
       return false;
     }
+  if (m_ucs4str[ci_offset] == ucs4) return true; // no change!
+  if (ci_offset == m_ucs4length)
+    {
+      throw std::out_of_range("UCS4String::iterator - attempt to set beyond end of string!");
+      return false;
+    }
 
   if (m_utf8str.byte_length ()) m_utf8str = ""; // internal UTF-8 string no longer valid
 
-  if (ucs4 & ~0x7fffffff) return false; // UCS-4 is 31-bit!
-
-  m_ucs4str[ci_offset] = ucs4;
-  return true;
+  if (ucs4 == 0) // hmm, truncation!
+    {
+      m_ucs4length = ci_offset;
+      m_ucs4str[ci_offset] = ucs4;
+      return true;
+    }
+  else if ((ucs4 & 0x7fffffff) == ucs4) // UCS-4 is 31-bit!
+    {
+      m_ucs4str[ci_offset] = ucs4;
+      return true;
+    }
+  return false;
 }
 
 const char * UCS4String::UCS4StringValue::utf8_str ()
